"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toTypeString = exports.toString = exports.algebra = void 0;
var _registry = /*#__PURE__*/require("@traversable/registry");
var _json = /*#__PURE__*/require("@traversable/json");
var _schema = /*#__PURE__*/require("@traversable/schema");
/** @internal */
const Object_entries = globalThis.Object.entries;
/** @internal */
const OPT = '<<>>';
/** @internal */
const trim = s => s == null ? String(s) : s.startsWith(OPT) ? s.substring(OPT.length) : s;
function typeName(x) {
  return x.tag.substring(_registry.NS.length);
}
var Recursive;
(function (Recursive) {
  const jsonToString = /*#__PURE__*/_json.Json.fold(x => {
    switch (true) {
      default:
        return _registry.fn.exhaustive(x);
      case _json.Json.isScalar(x):
        return typeof x === 'string' ? `"${x}"` : `${x}`;
      case _json.Json.isArray(x):
        return `[${x.join(',')}]`;
      case _json.Json.isObject(x):
        return `{ ${Object_entries(x).map(([k, v]) => `${(0, _registry.parseKey)(k)}: ${v}`).join(', ')} }`;
    }
  });
  Recursive.toString = x => {
    switch (true) {
      default:
        return _registry.fn.exhaustive(x);
      case _schema.t.isLeaf(x):
        return 't.' + typeName(x);
      case x.tag === _registry.URI.eq:
        return `t.eq(${jsonToString(x.def)})`;
      case x.tag === _registry.URI.array:
        return `t.${typeName(x)}(${x.def})`;
      case x.tag === _registry.URI.record:
        return `t.${typeName(x)}(${x.def})`;
      case x.tag === _registry.URI.optional:
        return `t.${typeName(x)}(${x.def})`;
      case x.tag === _registry.URI.tuple:
        return `t.${typeName(x)}(${x.def.join(', ')})`;
      case x.tag === _registry.URI.union:
        return `t.${typeName(x)}(${x.def.join(', ')})`;
      case x.tag === _registry.URI.intersect:
        return `t.${typeName(x)}(${x.def.join(', ')})`;
      case x.tag === _registry.URI.object:
        {
          const xs = Object_entries(x.def);
          return xs.length === 0 ? `t.${typeName(x)}({})` : `t.${typeName(x)}({ ${xs.map(([k, v]) => (0, _registry.parseKey)(k) + `: ${v}`).join(', ')} })`;
        }
    }
  };
  Recursive.toTypeString = x => {
    switch (true) {
      default:
        return _registry.fn.exhaustive(x);
      case _schema.t.isLeaf(x):
        return typeName(x);
      case x.tag === _registry.URI.eq:
        return jsonToString(x.def);
      case x.tag === _registry.URI.array:
        return `(${trim(x.def)})[]`;
      case x.tag === _registry.URI.record:
        return `Record<string, ${trim(x.def)}>`;
      case x.tag === _registry.URI.optional:
        return `${OPT}(${trim(x.def)} | undefined)`;
      case x.tag === _registry.URI.union:
        return `(${x.def.map(trim).join(' | ')})`;
      case x.tag === _registry.URI.intersect:
        return `(${x.def.map(trim).join(' & ')})`;
      case x.tag === _registry.URI.tuple:
        return `[${x.def.map(y => (y?.startsWith(OPT) ? '_?: ' : '') + trim(y)).join(', ')}]`;
      case x.tag === _registry.URI.object:
        {
          const xs = Object_entries(x.def);
          return xs.length === 0 ? `{}` : `{ ${xs.map(([k, v]) => (0, _registry.parseKey)(k) + (v?.startsWith(OPT) ? '?' : '') + `: ${trim(v)}`).join(', ')} }`;
        }
    }
  };
})(Recursive || (exports.algebra = Recursive = {}));
const toString = exports.toString = /*#__PURE__*/_schema.t.fold(Recursive.toString);
const toTypeString = schema => trim(_schema.t.fold(Recursive.toTypeString)(schema));
exports.toTypeString = toTypeString;
//# sourceMappingURL=recursive.js.map