"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.any = anyToString;
exports.array = arrayToString;
exports.bigint = bigintToString;
exports.boolean = booleanToString;
exports.eq = eqToString;
exports.integer = integerToString;
exports.intersect = intersectToString;
exports.never = neverToString;
exports.null = nullToString;
exports.number = numberToString;
exports.object = objectToString;
exports.optional = optionalToString;
exports.record = recordToString;
exports.string = stringToString;
exports.symbol = symbolToString;
exports.toString = toString;
exports.tuple = tupleToString;
exports.undefined = undefinedToString;
exports.union = unionToString;
exports.unknown = unknownToString;
exports.void = voidToString;
var _registry = /*#__PURE__*/require("@traversable/registry");
const Symbol_optional = _registry.symbol.optional;
/** @internal */
const isArray = globalThis.Array.isArray;
/** @internal */
const hasToString = x => !!x && typeof x === 'function' && 'toString' in x && typeof x.toString === 'function';
/** @internal */
const isOptional = u => !!u && typeof u === 'function' && Symbol_optional in u && typeof u[Symbol_optional] === 'number';
/** @internal */
const isShowable = u => u == null || typeof u === 'boolean' || typeof u === 'number' || typeof u === 'bigint' || typeof u === 'string';
/** @internal */
const stringify = u => typeof u === 'string' ? `'${u}'` : isShowable(u) ? globalThis.String(u) : 'string';
function toString(x) {
  return hasToString(x) ? x.toString() : 'unknown';
}
toString.never = 'never';
toString.unknown = 'unknown';
toString.any = 'any';
toString.void = 'void';
toString.undefined = 'undefined';
toString.null = 'null';
toString.symbol = 'symbol';
toString.boolean = 'boolean';
toString.integer = 'number';
toString.number = 'number';
toString.bigint = 'bigint';
toString.string = 'string';
toString.optional = x => `(${toString(x)} | undefined)`;
toString.array = x => '(' + toString(x) + ')[]';
toString.record = x => `Record<string, ${toString(x)}>`;
toString.eq = x => isShowable(typeof x) ? stringify(x) : typeof x === 'symbol' ? 'symbol' : 'string';
toString.union = xs => isArray(xs) ? xs.length === 0 ? 'never' : `(${xs.map(toString).join(' | ')})` : 'unknown';
toString.intersect = xs => isArray(xs) ? xs.length === 0 ? 'unknown' : `(${xs.map(toString).join(' & ')})` : 'unknown';
toString.tuple = xs => isArray(xs) ? `[${xs.map(x => isOptional(x) ? `_?: ${toString(x)}` : toString(x)).join(', ')}]` : 'unknown[]';
toString.object = xs => {
  if (!!xs && typeof xs === 'object') {
    const entries = Object.entries(xs);
    if (entries.length === 0) return '{}';else return `{ ${entries.map(([k, x]) => `'${k}${isOptional(x) ? "'?" : "'"}: ${toString(x)}`).join(', ')} }`;
  } else return '{ [x: string]: unknown }';
};
function neverToString() {
  return toString.never;
}
function unknownToString() {
  return toString.unknown;
}
function anyToString() {
  return toString.any;
}
function voidToString() {
  return toString.void;
}
function undefinedToString() {
  return toString.undefined;
}
function nullToString() {
  return toString.null;
}
function symbolToString() {
  return toString.symbol;
}
function booleanToString() {
  return toString.boolean;
}
function integerToString() {
  return toString.integer;
}
function bigintToString() {
  return toString.bigint;
}
function numberToString() {
  return toString.number;
}
function stringToString() {
  return toString.string;
}
function eqToString() {
  return toString.eq(this.def);
}
function arrayToString() {
  return toString.array(this.def);
}
function optionalToString() {
  return toString.optional(this.def);
}
function recordToString() {
  return toString.record(this.def);
}
function unionToString() {
  return toString.union(this.def);
}
function intersectToString() {
  return toString.intersect(this.def);
}
function tupleToString() {
  return toString.tuple(this.def);
}
function objectToString() {
  return toString.object(this.def);
}
//# sourceMappingURL=toString.js.map