import type { Returns, Join, Showable, UnionToTuple } from '@traversable/registry';
import { symbol } from '@traversable/registry';
import { t } from '@traversable/schema';
export { neverToString as never, unknownToString as unknown, anyToString as any, voidToString as void, nullToString as null, undefinedToString as undefined, symbolToString as symbol, booleanToString as boolean, integerToString as integer, bigintToString as bigint, numberToString as number, stringToString as string, eqToString as eq, arrayToString as array, recordToString as record, unionToString as union, intersectToString as intersect, optionalToString as optional, tupleToString as tuple, objectToString as object, };
declare const Symbol_optional: typeof symbol.optional;
export declare function toString(x: unknown): string;
export declare namespace toString {
    export var eq: <S>(x: S) => toString.eq<S>;
    export var intersect: <S>(xs: S) => toString.intersect<S>;
    export var union: <S>(xs: S) => toString.union<S>;
    export var record: <S>(x: S) => toString.record<S>;
    export var tuple: <S>(xs: S) => toString.tuple<S>;
    export var optional: <S>(x: S) => toString.optional<S>;
    export var array: <S>(x: S) => toString.array<S>;
    export var object: <S, _ = UnionToTuple<keyof S>>(xs: S) => toString.object<S, _>;
    export var never: "never";
    export var unknown: "unknown";
    export var any: "any";
    var _a: "void";
    export var undefined: "undefined";
    var _b: "null";
    export var symbol: "symbol";
    export var boolean: "boolean";
    export var integer: "number";
    export var number: "number";
    export var bigint: "bigint";
    export var string: "string";
    export { _a as void, _b as null };
}
export declare namespace toString {
    export type eq<T, _ extends string = [T] extends [Showable] ? `${T}` : never> = [_] extends [never] ? [T] extends [symbol] ? 'symbol' : string : [T] extends [string] ? `'${_}'` : _;
    export type intersect<T> = never | [T] extends [readonly []] ? 'unknown' : `(${Join<{
        [I in keyof T]: Returns<T[I]['toString']>;
    }, ' & '>})`;
    export type union<T> = never | [T] extends [readonly []] ? 'never' : `(${Join<{
        [I in keyof T]: Returns<T[I]['toString']>;
    }, ' | '>})`;
    export type record<T> = never | `Record<string, ${Returns<T['toString']>}>`;
    export type tuple<T> = never | `[${Join<{
        [I in keyof T]: `${T[I] extends {
            [Symbol_optional]: any;
        } ? `_?: ${Returns<T[I]['toString']>}` : Returns<T[I]['toString']>}`;
    }, ', '>}]`;
    export type optional<T> = never | `(${Returns<T['toString']>} | undefined)`;
    export type array<T> = `(${Returns<T['toString']>})[]`;
    export { object_ as object };
    export type object_<T, _ = UnionToTuple<keyof T>> = never | [keyof T] extends [never] ? '{}' : `{ ${Join<{
        [I in keyof _]: `'${_[I]}${T[_[I]] extends {
            [Symbol_optional]: any;
        } ? `'?` : `'`}: ${Returns<T[_[I]]['toString']>}`;
    }, ', '>} }`;
}
interface neverToString {
    toString(): typeof toString.never;
}
interface unknownToString {
    toString(): typeof toString.unknown;
}
interface anyToString {
    toString(): typeof toString.any;
}
interface voidToString {
    toString(): typeof toString.void;
}
interface nullToString {
    toString(): typeof toString.null;
}
interface undefinedToString {
    toString(): typeof toString.undefined;
}
interface bigintToString {
    toString(): typeof toString.bigint;
}
interface symbolToString {
    toString(): typeof toString.symbol;
}
interface booleanToString {
    toString(): typeof toString.boolean;
}
interface integerToString {
    toString(): typeof toString.integer;
}
interface numberToString {
    toString(): typeof toString.number;
}
interface stringToString {
    toString(): typeof toString.string;
}
declare function neverToString(): "never";
declare function unknownToString(): "unknown";
declare function anyToString(): "any";
declare function voidToString(): "void";
declare function undefinedToString(): "undefined";
declare function nullToString(): "null";
declare function symbolToString(): "symbol";
declare function booleanToString(): "boolean";
declare function integerToString(): "number";
declare function bigintToString(): "bigint";
declare function numberToString(): "number";
declare function stringToString(): "string";
interface eqToString<V = unknown> {
    toString(): Returns<typeof toString.eq<V>>;
}
interface arrayToString<S> {
    toString(): Returns<typeof toString.array<S>>;
}
interface optionalToString<S> {
    toString(): Returns<typeof toString.optional<S>>;
    [Symbol_optional]: number;
}
interface recordToString<S> {
    toString(): Returns<typeof toString.record<S>>;
}
interface unionToString<S> {
    toString(): Returns<typeof toString.union<S>>;
}
interface intersectToString<S> {
    toString(): Returns<typeof toString.intersect<S>>;
}
interface tupleToString<S> {
    toString(): Returns<typeof toString.tuple<S>>;
}
interface objectToString<S, _ = UnionToTuple<keyof S>> {
    toString(): Returns<typeof toString.object<S, _>>;
}
declare function eqToString<V>(this: t.eq<V>): toString.eq<V, [V] extends [Showable] ? `${V}` : never>;
declare function arrayToString<S>(this: t.array<S>): `(${Returns<S["toString"]>})[]`;
declare function optionalToString<S>(this: t.optional<S>): `(${Returns<S["toString"]>} | undefined)`;
declare function recordToString<S>(this: t.record<S>): `Record<string, ${Returns<S["toString"]>}>`;
declare function unionToString<S>(this: t.union<S>): toString.union<S>;
declare function intersectToString<S>(this: t.intersect<S>): toString.intersect<S>;
declare function tupleToString<S>(this: t.tuple<S>): `[${Join<{ [I in keyof S]: `${S[I] extends {
    [symbol_optional]: any;
} ? `_?: ${Returns<S[I]["toString"]>}` : Returns<S[I]["toString"]>}`; }, ", ", "">}]`;
declare function objectToString<S>(this: t.object<S>): toString.object<S, UnionToTuple.loop<[], keyof S, ((keyof S extends infer T ? T extends keyof S ? T extends T ? (_: () => T) => void : never : never : never) extends (_: infer _) => void ? _ : never) extends () => infer X ? X : never>>;
//# sourceMappingURL=toString.d.ts.map