import { fn, NS, parseKey, URI } from '@traversable/registry';
import { Json } from '@traversable/json';
import { t } from '@traversable/schema';
/** @internal */
const Object_entries = globalThis.Object.entries;
/** @internal */
const OPT = '<<>>';
/** @internal */
const trim = s => s == null ? String(s) : s.startsWith(OPT) ? s.substring(OPT.length) : s;
function typeName(x) {
  return x.tag.substring(NS.length);
}
export { Recursive as algebra };
var Recursive;
(function (Recursive) {
  const jsonToString = /*#__PURE__*/Json.fold(x => {
    switch (true) {
      default:
        return fn.exhaustive(x);
      case Json.isScalar(x):
        return typeof x === 'string' ? `"${x}"` : `${x}`;
      case Json.isArray(x):
        return `[${x.join(',')}]`;
      case Json.isObject(x):
        return `{ ${Object_entries(x).map(([k, v]) => `${parseKey(k)}: ${v}`).join(', ')} }`;
    }
  });
  Recursive.toString = x => {
    switch (true) {
      default:
        return fn.exhaustive(x);
      case t.isLeaf(x):
        return 't.' + typeName(x);
      case x.tag === URI.eq:
        return `t.eq(${jsonToString(x.def)})`;
      case x.tag === URI.array:
        return `t.${typeName(x)}(${x.def})`;
      case x.tag === URI.record:
        return `t.${typeName(x)}(${x.def})`;
      case x.tag === URI.optional:
        return `t.${typeName(x)}(${x.def})`;
      case x.tag === URI.tuple:
        return `t.${typeName(x)}(${x.def.join(', ')})`;
      case x.tag === URI.union:
        return `t.${typeName(x)}(${x.def.join(', ')})`;
      case x.tag === URI.intersect:
        return `t.${typeName(x)}(${x.def.join(', ')})`;
      case x.tag === URI.object:
        {
          const xs = Object_entries(x.def);
          return xs.length === 0 ? `t.${typeName(x)}({})` : `t.${typeName(x)}({ ${xs.map(([k, v]) => parseKey(k) + `: ${v}`).join(', ')} })`;
        }
    }
  };
  Recursive.toTypeString = x => {
    switch (true) {
      default:
        return fn.exhaustive(x);
      case t.isLeaf(x):
        return typeName(x);
      case x.tag === URI.eq:
        return jsonToString(x.def);
      case x.tag === URI.array:
        return `(${trim(x.def)})[]`;
      case x.tag === URI.record:
        return `Record<string, ${trim(x.def)}>`;
      case x.tag === URI.optional:
        return `${OPT}(${trim(x.def)} | undefined)`;
      case x.tag === URI.union:
        return `(${x.def.map(trim).join(' | ')})`;
      case x.tag === URI.intersect:
        return `(${x.def.map(trim).join(' & ')})`;
      case x.tag === URI.tuple:
        return `[${x.def.map(y => (y?.startsWith(OPT) ? '_?: ' : '') + trim(y)).join(', ')}]`;
      case x.tag === URI.object:
        {
          const xs = Object_entries(x.def);
          return xs.length === 0 ? `{}` : `{ ${xs.map(([k, v]) => parseKey(k) + (v?.startsWith(OPT) ? '?' : '') + `: ${trim(v)}`).join(', ')} }`;
        }
    }
  };
})(Recursive || (Recursive = {}));
export const toString = /*#__PURE__*/t.fold(Recursive.toString);
export const toTypeString = schema => trim(t.fold(Recursive.toTypeString)(schema));
//# sourceMappingURL=recursive.js.map