import { symbol } from '@traversable/registry';
export { neverToString as never, unknownToString as unknown, anyToString as any, voidToString as void, nullToString as null, undefinedToString as undefined, symbolToString as symbol, booleanToString as boolean, integerToString as integer, bigintToString as bigint, numberToString as number, stringToString as string, eqToString as eq, arrayToString as array, recordToString as record, unionToString as union, intersectToString as intersect, optionalToString as optional, tupleToString as tuple, objectToString as object };
const Symbol_optional = symbol.optional;
/** @internal */
const isArray = globalThis.Array.isArray;
/** @internal */
const hasToString = x => !!x && typeof x === 'function' && 'toString' in x && typeof x.toString === 'function';
/** @internal */
const isOptional = u => !!u && typeof u === 'function' && Symbol_optional in u && typeof u[Symbol_optional] === 'number';
/** @internal */
const isShowable = u => u == null || typeof u === 'boolean' || typeof u === 'number' || typeof u === 'bigint' || typeof u === 'string';
/** @internal */
const stringify = u => typeof u === 'string' ? `'${u}'` : isShowable(u) ? globalThis.String(u) : 'string';
export function toString(x) {
  return hasToString(x) ? x.toString() : 'unknown';
}
toString.never = 'never';
toString.unknown = 'unknown';
toString.any = 'any';
toString.void = 'void';
toString.undefined = 'undefined';
toString.null = 'null';
toString.symbol = 'symbol';
toString.boolean = 'boolean';
toString.integer = 'number';
toString.number = 'number';
toString.bigint = 'bigint';
toString.string = 'string';
toString.optional = x => `(${toString(x)} | undefined)`;
toString.array = x => '(' + toString(x) + ')[]';
toString.record = x => `Record<string, ${toString(x)}>`;
toString.eq = x => isShowable(typeof x) ? stringify(x) : typeof x === 'symbol' ? 'symbol' : 'string';
toString.union = xs => isArray(xs) ? xs.length === 0 ? 'never' : `(${xs.map(toString).join(' | ')})` : 'unknown';
toString.intersect = xs => isArray(xs) ? xs.length === 0 ? 'unknown' : `(${xs.map(toString).join(' & ')})` : 'unknown';
toString.tuple = xs => isArray(xs) ? `[${xs.map(x => isOptional(x) ? `_?: ${toString(x)}` : toString(x)).join(', ')}]` : 'unknown[]';
toString.object = xs => {
  if (!!xs && typeof xs === 'object') {
    const entries = Object.entries(xs);
    if (entries.length === 0) return '{}';else return `{ ${entries.map(([k, x]) => `'${k}${isOptional(x) ? "'?" : "'"}: ${toString(x)}`).join(', ')} }`;
  } else return '{ [x: string]: unknown }';
};
function neverToString() {
  return toString.never;
}
function unknownToString() {
  return toString.unknown;
}
function anyToString() {
  return toString.any;
}
function voidToString() {
  return toString.void;
}
function undefinedToString() {
  return toString.undefined;
}
function nullToString() {
  return toString.null;
}
function symbolToString() {
  return toString.symbol;
}
function booleanToString() {
  return toString.boolean;
}
function integerToString() {
  return toString.integer;
}
function bigintToString() {
  return toString.bigint;
}
function numberToString() {
  return toString.number;
}
function stringToString() {
  return toString.string;
}
function eqToString() {
  return toString.eq(this.def);
}
function arrayToString() {
  return toString.array(this.def);
}
function optionalToString() {
  return toString.optional(this.def);
}
function recordToString() {
  return toString.record(this.def);
}
function unionToString() {
  return toString.union(this.def);
}
function intersectToString() {
  return toString.intersect(this.def);
}
function tupleToString() {
  return toString.tuple(this.def);
}
function objectToString() {
  return toString.object(this.def);
}
//# sourceMappingURL=toString.js.map